@@ -166,7 +166,7 @@ typedef std::vector<TVarLivePair> TVarLiveVector;
 class TVarGatherTraverser : public TLiveTraverser {
 public:
     TVarGatherTraverser(const TIntermediate& i, bool traverseDeadCode, TVarLiveMap& inList, TVarLiveMap& outList, TVarLiveMap& uniformList)
-      : TLiveTraverser(i, traverseDeadCode, true, true, false)
+      : TLiveTraverser(i, traverseDeadCode, true, true, false, false)
       , inputList(inList)
       , outputList(outList)
       , uniformList(uniformList)
@@ -209,7 +209,7 @@ class TVarSetTraverser : public TLiveTraverser
 {
 public:
     TVarSetTraverser(const TIntermediate& i, const TVarLiveMap& inList, const TVarLiveMap& outList, const TVarLiveMap& uniformList)
-      : TLiveTraverser(i, true, true, true, false)
+      : TLiveTraverser(i, true, true, true, false, true)
       , inputList(inList)
       , outputList(outList)
       , uniformList(uniformList)
@@ -1019,6 +1019,7 @@ uint32_t TDefaultIoResolverBase::computeTypeLocationSize(const TType& type, EShL
 
 //TDefaultGlslIoResolver
 TResourceType TDefaultGlslIoResolver::getResourceType(const glslang::TType& type) {
+    assert(isValidGlslType(type));
     if (isImageType(type)) {
         return EResImage;
     }
@@ -1034,6 +1035,15 @@ TResourceType TDefaultGlslIoResolver::getResourceType(const glslang::TType& type
     if (isUboType(type)) {
         return EResUbo;
     }
+    if (isCombinedSamplerType(type)) {
+        return EResCombinedSampler;
+    }
+    if (isAsType(type)) {
+        return EResAs;
+    }
+    if (isTensorType(type)) {
+        return EResTensor;
+    }
     return EResCount;
 }
 
@@ -1383,6 +1393,7 @@ struct TDefaultIoResolver : public TDefaultIoResolverBase {
     bool validateBinding(EShLanguage /*stage*/, TVarEntryInfo& /*ent*/) override { return true; }
 
     TResourceType getResourceType(const glslang::TType& type) override {
+        assert(isValidGlslType(type));
         if (isImageType(type)) {
             return EResImage;
         }
@@ -1398,6 +1409,15 @@ struct TDefaultIoResolver : public TDefaultIoResolverBase {
         if (isUboType(type)) {
             return EResUbo;
         }
+        if (isCombinedSamplerType(type)) {
+            return EResCombinedSampler;
+        }
+        if (isAsType(type)) {
+            return EResAs;
+        }
+        if (isTensorType(type)) {
+            return EResTensor;
+        }
         return EResCount;
     }
 
@@ -1483,6 +1503,11 @@ struct TDefaultHlslIoResolver : public TDefaultIoResolverBase {
         if (isUboType(type)) {
             return EResUbo;
         }
+        // no support for combined samplers in HLSL
+        if (isAsType(type)) {
+            return EResAs;
+        }
+        // no support for tensors in HLSL
         return EResCount;
     }
 
