@@ -97,7 +97,6 @@ namespace glslang {
 /////////////////////////////////// Floating point constants: /////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////////////////////
 
-// 
 // Scan a single- or double-precision floating point constant.
 // Assumes that the scanner has seen at least one digit,
 // followed by either a decimal '.' or the letter 'e', or a
@@ -1226,7 +1225,9 @@ int TPpContext::tStringInput::scan(TPpToken* ppToken)
 //
 int TPpContext::tokenize(TPpToken& ppToken)
 {
-    for(;;) {
+    int stringifyDepth = 0;
+    TPpToken stringifiedToken; // Tokens are appended to this as they come in
+    for (;;) {
         int token = scanToken(&ppToken);
 
         // Handle token-pasting logic
@@ -1254,6 +1255,20 @@ int TPpContext::tokenize(TPpToken& ppToken)
         if (token == '\n')
             continue;
 
+        if (token == tStringifyLevelInput::PUSH) {
+            stringifyDepth++;
+            continue;
+        }
+        if (token == tStringifyLevelInput::POP) {
+            assert(stringifyDepth > 0);
+            stringifyDepth--;
+            if (stringifyDepth == 0) {
+                snprintf(ppToken.name, sizeof(ppToken.name), "%s", stringifiedToken.name);
+                return PpAtomConstString;
+            }
+            continue;
+        }
+
         // expand macros
         if (token == PpAtomIdentifier) {
             switch (MacroExpand(&ppToken, false, true)) {
@@ -1267,7 +1282,21 @@ int TPpContext::tokenize(TPpToken& ppToken)
             }
         }
 
+        bool needStringSupport = ifdepth == 0 && (token == PpAtomConstString || stringifyDepth > 0);
+        if (needStringSupport && parseContext.intermediate.getSource() != EShSourceHlsl) {
+            // HLSL allows string literals.
+            // GLSL allows string literals with GL_EXT_debug_printf.
+            const char* const string_literal_EXTs[] = { E_GL_EXT_debug_printf, E_GL_EXT_spirv_intrinsics };
+            parseContext.requireExtensions(ppToken.loc, 2, string_literal_EXTs, "string literal");
+            if (!parseContext.extensionTurnedOn(E_GL_EXT_debug_printf) &&
+                !parseContext.extensionTurnedOn(E_GL_EXT_spirv_intrinsics)) {
+                continue;
+            }
+        }
+
         switch (token) {
+        case PpAtomConstString:
+            break;
         case PpAtomIdentifier:
         case PpAtomConstInt:
         case PpAtomConstUint:
@@ -1281,17 +1310,6 @@ int TPpContext::tokenize(TPpToken& ppToken)
             if (ppToken.name[0] == '\0')
                 continue;
             break;
-        case PpAtomConstString:
-            // HLSL allows string literals.
-            // GLSL allows string literals with GL_EXT_debug_printf.
-            if (ifdepth == 0 && parseContext.intermediate.getSource() != EShSourceHlsl) {
-              const char* const string_literal_EXTs[] = { E_GL_EXT_debug_printf, E_GL_EXT_spirv_intrinsics };
-              parseContext.requireExtensions(ppToken.loc, 2, string_literal_EXTs, "string literal");
-              if (!parseContext.extensionTurnedOn(E_GL_EXT_debug_printf) &&
-                  !parseContext.extensionTurnedOn(E_GL_EXT_spirv_intrinsics))
-                  continue;
-            }
-            break;
         case '\'':
             parseContext.ppError(ppToken.loc, "character literals not supported", "\'", "");
             continue;
@@ -1299,6 +1317,17 @@ int TPpContext::tokenize(TPpToken& ppToken)
             snprintf(ppToken.name, sizeof(ppToken.name), "%s", atomStrings.getString(token));
             break;
         }
+        if (stringifyDepth > 0) {
+            size_t existingLen = strlen(stringifiedToken.name);
+            char* dst = stringifiedToken.name + existingLen;
+            // stringify_depth would determine how many layers of \\\"\\\" are needed, if we wanted to.
+            if (ppToken.space) {
+                snprintf(dst, sizeof(stringifiedToken.name) - existingLen - 1, " %s", ppToken.name);
+            } else {
+                snprintf(dst, sizeof(stringifiedToken.name) - existingLen, "%s", ppToken.name);
+            }
+            continue;
+        }
 
         return token;
     }
@@ -1329,6 +1358,7 @@ int TPpContext::tokenPaste(int token, TPpToken& ppToken)
 
         // This covers end of macro expansion
         if (endOfReplacementList()) {
+            // this should be unreachable, incomplete #/## sequences are caught at macro parsing time.
             parseContext.ppError(ppToken.loc, "unexpected location; end of replacement list", "##", "");
             break;
         }
