@@ -184,7 +184,10 @@ INSTANTIATE_TEST_SUITE_P(
         "aggOps.frag",
         "always-discard.frag",
         "always-discard2.frag",
+        "boolinput.error.frag",
+        "booloutput.error.vert",
         "conditionalDiscard.frag",
+        "conservativeDepth.frag",
         "conversion.frag",
         "dataOut.frag",
         "dataOutIndirect.frag",
@@ -213,6 +216,7 @@ INSTANTIATE_TEST_SUITE_P(
         "runtimeArray.vert",
         "simpleFunctionCall.frag",
         "stringToDouble.vert",
+        "stringInvalidOps.frag",
         "struct.error.frag",
         "structAssignment.frag",
         "structDeref.frag",
